"""Update data structures code"""

from soil.data_structures.es_data_structure import ESDataStructure
from soil.data_structures.predictions import Predictions

print("updating code - !")
print(Predictions, ESDataStructure)
